/* ================================================================ */
/*  Rob Hamerling's MAXIMUS download file scan and sort utility     */
/*  -> DOWNRPT4.C                                                   */
/*  -> Make GBL-list, ALL-list.                                     */
/* ================================================================ */

#define INCL_BASE
#define INCL_NOPMAPI
#include <os2.h>

#include <conio.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "..\max\mstruct.h"
#include "downsort.h"
#include "downfpro.h"

/* prototypes of local functions */
/* ------------------------------------------- */
/* Produce the file-request format of GBL-list */
/* ------------------------------------------- */
void make_gbl(FILECHAIN * _HUGE *dm,
              DOWNPATH  _HUGE *area,
              USHORT    x)              /* report privilege index        */
{
  FILE   *pf;                           /* file handle                   */
  char   outfile[MAXFN];                /* file names                    */
  ULONG  i,j,fc;                        /* counters                      */
  ULONG  akb;                           /* bytecount in KB               */
  FILECHAIN *cn;                        /* pointer to info of spec. file */
  FILECHAIN *fx;                        /* pointer to file info          */

  sprintf(outfile,"%s.%s%c",
          lp[P_GBL].name,
          lp[P_GBL].ext,
          priv_name[lp[P_GBL].priv[x]-TWIT][0]);
  pf = fopen(outfile,WRITE);            /* output file                   */
  if (pf != NULL) {
    if (oper_mode == VERBOSE)
      fprintf(stdout, MSG_SRT, file_total_count, area_total_count, outfile);
    switch(lp[P_GBL].sortflag) {
      case ALPHA:     psort(dm,0,file_total_count-1,sort_gbl); break;
      case TIMESTAMP: psort(dm,0,file_total_count-1,sort_new); break;
      default: break;
      }
    fc = preproc_area(area, dm, lp[P_GBL].priv[x]);
    cn = NULL;                          /* no assigned                   */
    for (i=0; i<file_total_count; ++i) {      /* stop at end of files    */
      fx = dm[i];                       /* copy pointer to file info */
      if (fx->priv <= lp[P_GBL].priv[x])   /* check file-privilege    */
        cn = new_acq(fx, cn);           /* keep pointer to most recent   */
      }
    if (oper_mode != QUIET)
      fprintf(stdout, MSG_REP, outfile);
    if (oper_mode==VERBOSE)
      fprintf(stdout, MSG_REC);
    insert_title(pf, pre_title, 0);
    block_title(pf, 20, list_title, P_GBL);
    file_incl(pf, P_GBL);               /* insert user-'logo'            */
    insert_title(pf, sub_title, 0);
    akb = (count_bytes(area) + 512)/1024;    /* KBytes total */
    fprintf(pf,"\n(%s) Available: %lu files (%lu %cB)",
                sys_date(today),
                fc,
                (akb < 9999) ? akb : (akb + 512)/1024,
                (akb < 9999) ? 'K' : 'M');
    if (lp[P_GBL].exclflag != EXCLPRIV)
      fprintf(pf,"\n%19s%s%s",
               "", MP, priv_name[lp[P_GBL].priv[x]-TWIT]);
    if (cn != NULL) {
      fprintf(pf,"\n%19sNewest: %s %8s",
               "",cn->fname,f_date(cn->wdate));
      fprintf(pf," (avail: %8s)",f_date(cn->cdate));
      }
    fprintf(pf,"\n%19s%s %c%s, %c%s",
               "", DF, DAYS_7, WK, DAYS_30, MO);
    fprintf(pf,"\n\n%s       %s    %s   %s    %s\n",FN,AC,SZ,DT,DS);
    sep_line(pf, '', 12, 8, 5, 9, 41, 0);
    for (i=j=0; i<file_total_count; i++) {
      fx = dm[i];                       /* copy pointer to file info */
      if (fx->priv <= lp[P_GBL].priv[x]) {   /* within priv limit     */
        if (oper_mode==VERBOSE && (j%25)==0) {
          fprintf(stdout, "%6lu\r", j);
          fflush(stdout);
          }
        if (fx->fname[0] != '\0') {  /* not a comment-entry           */
          j++;
          fprintf(pf,"%-12.12s %-8.8s %15s ",
                  fx->fname,
                  fx->parea->name,
                  f_size_date(fx->size, fx->wdate,
                              fx->cdate, fx->ctime));
          desc_part(pf, fx->fdesc, 41, 41-lp[P_GBL].desc_indent, P_GBL);
          }
        }
      }
    if (oper_mode==VERBOSE)
      fprintf(stdout, "%6lu\n", j);
    signature(pf,today);                /* leave fingerprint             */
    insert_title(pf, bot_lines, 0);
    fclose(pf);                         /* finished with .GBL file       */
    }
  else
    fprintf(stderr, MSG_OPO, outfile, 0);      /* no GBL list!         */
  }

/* ------------------------------------------------------- */
/* Produce the file-request format of ALLfiles  (ALL-list) */
/* ------------------------------------------------------- */
void make_all(FILECHAIN * _HUGE *dm,
              DOWNPATH  _HUGE *area,
              USHORT    x)
{
  FILE     *pf;                         /* file handle                   */
  char     outfile[MAXFN];              /* file names                    */
  char     ac[40];                      /* area name                     */
  ULONG    i, n, fc;                    /* counters                      */
  ULONG    akb;                         /* bytecount in KB               */
  USHORT   j;                           /* counters                      */
  DOWNPATH _HUGE *area2;                /* copy of area-array for sort   */
  FILECHAIN *fx;                        /* pointer to file info          */

  sprintf(outfile,"%s.%s%c",
          lp[P_ALL].name,
          lp[P_ALL].ext,
          priv_name[lp[P_ALL].priv[x]-TWIT][0]);
  pf = fopen(outfile,WRITE);            /* output file                   */
  if (pf != NULL) {
    if (oper_mode == VERBOSE)
      fprintf(stdout, MSG_SRT, file_total_count, area_total_count, outfile);
    switch(lp[P_ALL].sortflag) {
      case ALPHA:     psort(dm,0,file_total_count-1,sort_all); break;
      case TIMESTAMP: psort(dm,0,file_total_count-1,sort_al2); break;
      case KEEPSEQ:   psort(dm,0,file_total_count-1,sort_akp); break;
      default: break;
      }

    fc = preproc_area(area, dm, lp[P_ALL].priv[x]);
    if (oper_mode!=QUIET)
      fprintf(stdout, MSG_REP, outfile);
    if (oper_mode==VERBOSE)
      fprintf(stdout, MSG_REC);
    insert_title(pf, pre_title, 0);
    block_title(pf, 20, list_title, P_ALL);
    file_incl(pf, P_ALL);               /* insert user-'logo'            */
    insert_title(pf, sub_title, 0);
    akb = (count_bytes(area) + 512)/1024;    /* KBytes total */
    fprintf(pf,"\n(%s) Available: %lu files in %hu areas (%lu %cB)\n",
               sys_date(today),
               fc,
               count_areas(area, lp[P_ALL].priv[x]),
               (akb < 9999) ? akb : (akb + 512)/1024,
               (akb < 9999) ? 'K' : 'M');
    if (lp[P_ALL].exclflag != EXCLPRIV)
      fprintf(pf,"%19s%s%s\n", "",MP,priv_name[lp[P_ALL].priv[x]-TWIT]);
    fprintf(pf,"%19s%s %c%s, %c%s\n",
               "", DF, DAYS_7, WK, DAYS_30, MO);
    ac[0] = '\0';                       /* null area name                */
    n=0;                                /* file counter for SYSOP        */
    for (i=0; i<file_total_count; i++) {    /* all files                 */
      fx = dm[i];                       /* copy pointer to file info */
      if (strcmp(ac, fx->parea->name)) {  /* new area collection       */
        strcpy(ac, fx->parea->name);     /* set new                   */
        if (fx->parea->file_count > 0) { /* any listable files in area*/
          fprintf(pf,"\n\n");
          sep_line(pf, '', 79, 0);
          akb = (fx->parea->byte_count + 512)/1024;
          if (max_aname <= 3) {          /* short areanames              */
            fprintf(pf,"%s  %-.60s\n",
                        strnblk(ac,3,FONT3,LINE1),fx->parea->adesc);
            fprintf(pf,"%s  Available: %hu files (%lu %cB)\n",
                      strnblk(ac,3,FONT3,LINE2),
                      fx->parea->file_count,
                      (akb < 9999) ? akb : (akb + 512)/1024,
                      (akb < 9999) ? 'K' : 'M');
            fprintf(pf,"%s ",
                        strnblk(ac,3,FONT3,LINE3));
            if (lp[P_ALL].exclflag != EXCLPRIV)
              fprintf(pf," Privilege: %-.9s\n",
                        priv_name[fx->parea->priv-TWIT]); /* area priv*/
            else
              fprintf(pf,"\n");
            fprintf(pf,"%s ",strnblk(ac,3,FONT3,LINE4));
            }
          else {                        /* long areanames                */
            fprintf(pf,"%s\n", strnblk(ac,8,FONT3,LINE1));
            fprintf(pf,"%s\n", strnblk(ac,8,FONT3,LINE2));
            fprintf(pf,"%s\n", strnblk(ac,8,FONT3,LINE3));
            fprintf(pf,"%s\n", strnblk(ac,8,FONT3,LINE4));
            sep_line(pf, '', 79, 0);
            fprintf(pf," %-.78s\n", fx->parea->adesc);
            fprintf(pf," Available: %hu files (%lu %cB)\n",
                     fx->parea->file_count,
                     (akb < 9999) ? akb : (akb + 512)/1024,
                     (akb < 9999) ? 'K' : 'M');
            if (lp[P_ALL].exclflag != EXCLPRIV)
              fprintf(pf," Privilege: %-.9s\n",
                        priv_name[fx->parea->priv-TWIT]); /* area priv*/
            }
          if (fx->parea->newest != NULL) {     /* newest file         */
            fprintf(pf," Newest: %s %8s ",
                        fx->parea->newest->fname,
                        f_date(fx->parea->newest->wdate));
            fprintf(pf," (avail: %8s)",
                        f_date(fx->parea->newest->cdate));
            }
          fprintf(pf, "\n");
          sep_line(pf, '', 79, 0);
          fprintf(pf,"%s      %s   %s    %s\n",FN,SZ,DT,DS);
          sep_line(pf, '', 12, 5, 9, 50, 0);
          }
        }
      if (fx->priv <= lp[P_ALL].priv[x]) {  /* only upto max priv_lvl */
        if (oper_mode==VERBOSE && (n%25)==0) {
          fprintf(stdout, "%6lu\r", n);  /* keep SYSOP awake             */
          fflush(stdout);
          }
        if (fx->fname[0] != '\0') {  /* filename present              */
          ++n;                          /* count only file entries       */
          fprintf(pf,"%-12.12s %15s ",
                     fx->fname,
                     f_size_date(fx->size,  fx->wdate,
                                 fx->cdate, fx->ctime));
          desc_part(pf, fx->fdesc, 50, 50-lp[P_ALL].desc_indent, P_ALL);
          }
        else if (lp[P_ALL].sortflag == KEEPSEQ) {  /* '/K' specified     */
          fprintf(pf, "%-s\n",
            (strip_ava == 'Y') ? strava(fx->fdesc) : fx->fdesc);
          }
        }
      }
    if (oper_mode==VERBOSE)             /* last value                    */
      fprintf(stdout, "%6lu\n", n);
                                        /* Area Summary report           */
#ifndef __32BIT__
    area2 = (DOWNPATH huge *)halloc(area_total_count, sizeof(DOWNPATH));
#else
    area2 = (DOWNPATH *)malloc(area_total_count * sizeof(DOWNPATH));
#endif
                                        /* dup area-array for sorting!   */
    if (area2 == NULL) {                /* memory not obtained?          */
      if (oper_mode!=QUIET)
        fprintf(stderr, "Not enough memory for summary report, skipped.\n");
      }
    else {
      for (j=0; j<area_total_count; j++)
        memmove(&area2[j],&area[j],sizeof(DOWNPATH));
      qsort(area2, area_total_count, sizeof(DOWNPATH), sort_summ);
      if (oper_mode==VERBOSE)
        fprintf(stdout, MSG_REP, SUMM);
      fprintf(pf,"\n\n");
      sep_line(pf, '', 79, 0);
      block_title(pf, 12, SUMM, P_ALL);
      sep_line(pf, '', 79, 0);
      fprintf(pf,"%-8s %-54s %5s %8s\n",AC,DS,FS,BY);
      sep_line(pf, '', 8, 54, 5, 9, 0);
      for (j=0; j<area_total_count; j++)
        if (area2[j].file_count) {
          fprintf(pf,"%-8.8s %-54.54s %5u %8luK\n",
                     area2[j].name,
                     area2[j].adesc,
                     area2[j].file_count,
                     (area2[j].byte_count+512)/1024);
          }
      sep_line(pf, '', 63, 5, 9, 0);
      fprintf(pf,"%63s %5lu %8luK\n","Total:",
                 count_files(area2),
                 (count_bytes(area2)+512)/1024);
#ifndef __32BIT__
      hfree(area2);                     /* free copy of area array       */
#else
      free(area2);                      /* free copy of area array       */
#endif
      }

    signature(pf,today);                /* leave fingerprint             */
    insert_title(pf, bot_lines, 0);
    fclose(pf);                         /* finished with .ALL file       */
    }
  else
    fprintf(stderr, MSG_OPO, outfile, 0);      /* open failed          */
  }

