* This is a file containing two functions 
*   GetDoorInf()    for reading info from the door.sys file
*   GetUserInf()    for reading info from the userinfo.sys file

* Cut and paste these modules as needed.
* Any comments or suggestions, please direct to

* Robin Bayer   
* Ultimate Solutions
* P.O. Box 11513
* Overland Park, KS 66207-4213

func GetDoorInf
* Name    : GetDoorInf (procedure)
* Author  : Robin J. Bayer
* Date    : 93-07-25
* Purpose : This procedure will attempt to read the information stored
*           in the DOOR.SYS files for use with Wildcat! 3.x doors  
*           (and other BBS's using Door.sys) This information will be 
*           put into public memvars.  Be sure to check the names of memvars 
*           defined in this module and change them if a conflict occurs.
* Given   : m.filepath (the path to the door.sys file)
* Returns : .t. if all information was successfully read, .f. if not

  para m.filepath           && path where files are located

  if type('m.filepath') != 'C'   && must be character type
    wait wind 'A character string path must be passed to this function' ;
              
    return .f.
  endif
  
  priv m.ReturnVal          && return value
  priv m.fhandle            && file handle to open door sys
  
  * Declare door.sys memvars public
  publ m.wport, m.wbaud, m.wparity, m.wnode, m.wDTErate, m.wscreen, ;
       m.wprinter, m.wpagebell, m.walarm, m.wfullname, m.wcallfrom, ;
       m.whomephone, m.wworkphone, m.wpassword, m.wseclevel, ;
       m.wtottimeon, m.wdatelast, m.wsremain, m.wmremain, m.wgraphmode, ;
       m.wpagelen, m.wusermode, m.wconfreg, m.wconfexit, m.wexpdate, ;
       m.wUserRecno, m.wdefprot, m.wtotup, m.wtotdn, m.wdaydntot, ;
       m.wdaydnmax, m.wuserbdate, m.wmainpath, m.wgenpath, m.wsysop, ;
       m.waliasname, m.weventtime, m.werrorcor, m.wansi, m.wreclock, ;
       m.wbbscolor, m.wtimecr, m.wlastscan, m.wtimecall, m.wlastcall, ;
       m.wmaxdayfil, m.wdaydnsfar, m.wtotupkb, m.wtotdnkb, m.wcomment, ;
       m.wtotdoors, m.wtotmsgs

  * Now store empty values in order to establish type as well as
  * to have an empty value if a false return occurs
  
  m.wport      = space(4)      && comm port
  m.wbaud      = 0             && baud rate
  m.wparity    = 8             && parity  (default to 8)
  m.wnode      = 0             && node number
  m.wDTErate   = 0             && DTE rate
  m.wscreen    = .f.           && screen display ?
  m.wprinter   = .f.           && printer toggle ?
  m.wpagebell  = .f.           && page bell ?
  m.walarm     = .f.           && caller alarm ?
  m.wfullname  = space(64)     && caller full name
  m.wcallfrom  = space(64)     && caller from field
  m.whomephone = space(12)     && caller home phone
  m.wworkphone = space(12)     && caller work/data phone
  m.wpassword  = space(18)     && password
  m.wseclevel  = space(10)     && user security level
  m.wtottimeon = 0             && total times on
  m.wdatelast  = {  /  /  }    && date last on
  m.wsremain   = 0             && seconds remaining this call
  m.wmremain   = 0             && minutes remaining this call
  m.wgraphmode = '  '          && graphics mode
  m.wpagelen   = 0             && page length
  m.wusermode  = ' '           && usermode (Y = Expert, N = Novice)
  m.wconfreg   = space(20)     && conferences registered in
  m.wconfexit  = space(4)      && conference exited to door in
  m.wexpdate   = {  /  /  }    && user expiration date
  m.wUserRecno = 0             && user record number
  m.wdefprot   = ' '           && default protocol
  m.wtotup     = 0             && total uploads
  m.wtotdn     = 0             && total downloads
  m.wdaydntot  = 0             && daily download k total
  m.wdaydnmax  = 0             && daily download k max
  m.wuserbdate = {  /  /  }    && user birthdate
  m.wmainpath  = space(79)     && path to main directory (user file)
  m.wgenpath   = space(79)     && path to gen directory
  m.wsysop     = space(64)     && sysop name
  m.waliasname = space(64)     && alias name
  m.weventtime = space(5)      && event time hh:mm
  m.werrorcor  = .f.           && error correcting modem ?
  m.wansi      = .f.           && ansi supported and user in NG mode ?
  m.wreclock   = .f.           && use record locking ?
  m.wbbscolor  = 0             && BBS default color (IBM color mode)
  m.wtimecr    = 0             && time credits in minutes
  m.wlastscan  = {  /  /  }    && date of last new files scan
  m.wtimecall  = space(5)      && time of this call hh:mm
  m.wlastcall  = space(5)      && time of last call (usually 2:00AM!)
  m.wmaxdayfil = 0             && maximun daily files available
  m.wdaydnsfar = 0             && files dnld so far today
  m.wtotupkb   = 0             && total k bytes uploaded
  m.wtotdnkb   = 0             && total k bytes downloaded
  m.wcomment   = space(79)     && user comment
  m.wtotdoors  = 0             && total doors opened
  m.wtotmsgs   = 0             && total messages left

  m.returnval = .t.     && assume positive return until error
  
  m.filepath = alltrim(m.filepath)  && trim leading and trailing blanks
  if right(m.filepath,1) != '\'
    m.filepath = m.filepath + '\'
  endif
  m.fhandle = fopen(m.filepath+'door.sys',0)   && open readonly 
  if (m.fhandle > 0)  && opened sucessfully
    m.wport = left(fgets(m.fhandle),4)
    m.wbaud = val(left(fgets(m.fhandle),6))
    m.wparity = val(left(fgets(m.fhandle),1))
    m.wnode = val(left(fgets(m.fhandle),3))
    m.wDTErate = val(left(fgets(m.fhandle),6))
    if left(fgets(m.fhandle),1) = 'Y'
      m.wscreen = .t.
    else
      m.wscreen = .f.
    endif    
    if left(fgets(m.fhandle),1) = 'Y'
      m.wprinter = .t.
    else
      m.wprinter = .f.
    endif    
    if left(fgets(m.fhandle),1) = 'Y'
      m.wpagebell = .t.
    else
      m.wpagebell = .f.
    endif    
    if left(fgets(m.fhandle),1) = 'Y'
      m.walarm = .t.
    else
      m.walarm = .f.
    endif    
    m.wfullname = left(fgets(m.fhandle),64)
    m.wcallfrom = left(fgets(m.fhandle),64)
    m.whomephone = left(fgets(m.fhandle),12)
    m.wworkphone = left(fgets(m.fhandle),12)
    m.wpassword = left(fgets(m.fhandle),18)
    m.wseclevel = left(fgets(m.fhandle),10)
    m.wtottimeon = val(left(fgets(m.fhandle),5))
    m.wdatelast = ctod(left(fgets(m.fhandle),8))
    m.wsremain = val(left(fgets(m.fhandle),5))
    m.wmremain = val(left(fgets(m.fhandle),5))
    m.wgraphmode = left(fgets(m.fhandle),2)
    m.wpagelen = val(left(fgets(m.fhandle),2))
    m.wusermode = left(fgets(m.fhandle),2)
    m.wconfreg = left(fgets(m.fhandle),20)
    m.wconfexit = left(fgets(m.fhandle),4)
    m.wexpdate = ctod(left(fgets(m.fhandle),8))
    m.wUserRecno = val(left(fgets(m.fhandle),5))
    m.wdefprot = left(fgets(m.fhandle),1)
    m.wtotup = val(left(fgets(m.fhandle),5))
    m.wtotdn = val(left(fgets(m.fhandle),5))
    m.wdaydntot = val(left(fgets(m.fhandle),9))
    m.wdaydnmax = val(left(fgets(m.fhandle),9))
    m.wuserbdate = ctod(left(fgets(m.fhandle),8))
    m.wmainpath = left(fgets(m.fhandle),79)
    m.wgenpath = left(fgets(m.fhandle),79)
    m.wsysop = left(fgets(m.fhandle),64)
    m.waliasname = left(fgets(m.fhandle),64)
    m.weventtime = left(fgets(m.fhandle),5)
    if left(fgets(m.fhandle),1) = 'Y'
      m.werrorcor = .t.
    else
      m.werrorcor = .f.
    endif    
    if left(fgets(m.fhandle),1) = 'Y'
      m.wansi = .t.
    else
      m.wansi = .f.
    endif    
    if left(fgets(m.fhandle),1) = 'Y'
      m.wreclock = .t.
    else
      m.wreclock = .f.
    endif    
    m.wbbscolor = val(left(fgets(m.fhandle),2))
    m.wtimecr = val(left(fgets(m.fhandle),5))
    m.wlastscan = ctod(left(fgets(m.fhandle),8))
    m.wtimecall = left(fgets(m.fhandle),5)
    m.wlastcall = left(fgets(m.fhandle),5)
    m.wmaxdayfil = val(left(fgets(m.fhandle),5))
    m.wdaydnsfar = val(left(fgets(m.fhandle),5))
    m.wtotupkb = val(left(fgets(m.fhandle),9))
    m.wtotdnkb = val(left(fgets(m.fhandle),9))
    m.wcomment = left(fgets(m.fhandle),79)
    m.wtotdoors = val(left(fgets(m.fhandle),5))
    m.wtotmsgs = val(left(fgets(m.fhandle),5))
    =fclose(m.fhandle)   && close file when we are done

  else  && file did not open correctly.  Case will determine reason
    do case
      case (ferror() = 2)
        wait wind 'Door.sys file not found in specified path' 
      
      case (ferror() = 4)
        wait wind 'Too many files open (not enough handles)' 
      
      case (ferror() = 5)
        wait wind 'Access denied'
      
      case (ferror() = 6)
        wait wind 'Invalid file handle given'
      
      case (ferror() = 8)
        wait wind 'Out of memory' 
     
      case (ferror() = 31)
        wait wind 'Error opening file or general failure'
        
    otherwise
      wait wind 'Unspecified error opening file' 
    endcase
    =fclose(m.fhandle)
    return .f.
  endif      


func GetUserInf
* Name    : GetUserInf (procedure)
* Author  : Robin J. Bayer
* Date    : 93-07-26
* Purpose : This procedure will attempt to read the information stored
*           in the USERINFO.DAT files for use with Wildcat! 3.x doors  
*           This information will be put into public memvars.  Be sure 
*           to check the names of memvars defined in this module and 
*           change them if a conflict occurs.
* Given   : m.filepath (the path to the USERINFO.DAT file)
* Returns : .t. if all information was successfully read, .f. if not

  para m.filepath           && path where files are located

  if type('m.filepath') != 'C'   && must be character type
    wait wind 'A character string path must be passed to this function' ;
              
    return .f.
  endif
  
  priv m.ReturnVal          && return value
  priv m.fhandle            && file handle to open door sys
  
  * Declare userinfo.dat memvars public
  publ m.wfname, m.wconnect, m.wdatabit, m.wansienab, m.wmnpcnct, ;
       m.wtdofcall, m.wtddoor, m.wminremn, m.wcmdline, m.wcurconf, ;
       m.wactmenu, m.wsecurity, m.wbanktime, m.wdoorno, m.waname, ;
       m.whangup, m.wfdnld, m.wkbdnld

  * store empty values to memvars to declare type
        
  m.wfname     = space(64)   && full name
  m.wconnect   = space(6)    && needs to be character for LOCAL
  m.wdatabit   = 0           && data bits
  m.wansienab  = .f.         && ansi enabled
  m.wwmnpcnct  = .f.         && MNP connect
  m.wtdofcall  = space(14)   && time and date of call
  m.wtddoor    = space(14)   && time and date entered door
  m.wminremn   = 0           && minutes remaining
  m.wcmdline   = .f.         && command line switches?
  m.wcurconf   = ' '         && current conference
  m.wactmenu   = 0           && active menu 0=main,1=msg,2=file,3=sysop
  m.wsecurity  = space(10)   && security level name
  m.wbanktime  = 0           && banked time (MaxLogOn)
  m.wdoorno    = 0           && door name
  m.waname     = space(64)   && alias name
  m.whangup    = .f.         && did user hang up in door ?
  m.wfdnld     = 0           && files downloaded
  m.wwkbdnld   = 0           && kb downloaded
   
  m.filepath = alltrim(m.filepath)  && trim leading and trailing blanks
  if right(m.filepath,1) != '\'
    m.filepath = m.filepath + '\'
  endif
  m.fhandle = fopen(m.filepath+'userinfo.dat',0)   && open readonly 
  if (m.fhandle > 0)  && opened sucessfully
    m.wfname = left(fgets(m.fhandle),64)
    m.wconnect = left(fgets(m.fhandle),6)
    m.wdatabit = val(left(fgets(m.fhandle),1))
    if left(fgets(m.fhandle),1) = 'Y'
      m.wansienab = .t.
    else
      m.wansienab = .f.
    endif    
    if left(fgets(m.fhandle),1) = 'Y'
      m.wmnpcnct = .t.
    else
      m.wmnpcnct = .f.
    endif    
    m.wtdofcall = left(fgets(m.fhandle),14)
    m.wtddoor = left(fgets(m.fhandle),14)
    m.wminremn = val(left(fgets(m.fhandle),6))
    if left(fgets(m.fhandle),1) = 'Y'
      m.wcmdline = .t.
    else
      m.wcmdline = .f.
    endif    
    m.wcurconf = left(fgets(m.fhandle),1)
    m.wactmenu = val(left(fgets(m.fhandle),1))
    m.wsecurity = left(fgets(m.fhandle),10)
    m.wbanktime = val(left(fgets(m.fhandle),5))
    m.wdoorno = val(left(fgets(m.fhandle),2))
    m.waname = left(fgets(m.fhandle),64)
    if left(fgets(m.fhandle),1) = 'Y'
      m.whangup = .t.
    else
      m.whangup = .f.
    endif    
    m.wfdnld = val(left(fgets(m.fhandle),5))
    m.wwkbdnld = val(left(fgets(m.fhandle),9))
    =fclose(m.fhandle)   && close file when we are done

  else  && file did not open correctly.  Case will determine reason
    do case
      case (ferror() = 2)
        wait wind 'Userinfo.dat file not found in specified path' 
      
      case (ferror() = 4)
        wait wind 'Too many files open (not enough handles)' 
      
      case (ferror() = 5)
        wait wind 'Access denied'
      
      case (ferror() = 6)
        wait wind 'Invalid file handle given'
      
      case (ferror() = 8)
        wait wind 'Out of memory' 
     
      case (ferror() = 31)
        wait wind 'Error opening file or general failure'
        
    otherwise
      wait wind 'Unspecified error opening file' 
    endcase
    =fclose(m.fhandle)
    return .f.
  endif      
